from fastapi.requests import Request
from fastapi.responses import JSONResponse


class LinePayException(Exception):
    def __init__(self, error: str, detail: str = "") -> None:
        super().__init__(error)
        self.error = error
        self.detail = detail


def linepay_exception_handler(request: Request, exc: LinePayException) -> JSONResponse:
    return JSONResponse(
        status_code=400,
        content={"error": exc.error, "detail": exc.detail},
    )



import base64
import hmac
import json
import uuid
from hashlib import sha256
from typing import Any, Dict, List, Optional

import requests

from fastel.payment.linepay.exceptions import LinePayException


class LinePay:
    @property
    def linepay_url(self) -> str:
        if self.stage in ["prod", "PROD"]:
            return "https://api-pay.line.me"
        return "https://sandbox-api-pay.line.me"

    def __init__(
        self,
        stage: str,
        channel_id: str,
        channel_secret: str,
        callback_url: str,
    ):
        self.stage = stage
        self.channel_id = channel_id
        self.channel_secret = channel_secret
        self.callback_url = callback_url

    def build_headers(
        self,
        uri: str,
        request_body: str,
    ) -> Dict[str, Any]:
        nonce = str(uuid.uuid4())
        _signature = hmac.new(
            key=self.channel_secret.encode(),
            msg=(self.channel_secret + uri + request_body + nonce).encode(),
            digestmod=sha256,
        )
        signature = base64.b64encode(_signature.digest()).decode()
        return {
            "Content-Type": "application/json",
            "X-LINE-ChannelId": self.channel_id,
            "X-LINE-Authorization-Nonce": nonce,
            "X-LINE-Authorization": signature,
        }

    def _raise_exception(self, resp: requests.Response) -> None:
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as err:
            raise LinePayException(error="linepay_request_error", detail=str(err))

    def verify(self, response: Dict[str, Any]) -> bool:
        if response["returnCode"] == "0000":
            return True
        return False

    def _to_products(self, checkout_dict: Dict[str, Any]) -> List[Dict[str, Any]]:
        # only display one item for total order amount
        first_item = checkout_dict["items"][0]
        return [
            {
                "id": uuid.uuid4().__str__(),
                "amount": checkout_dict["total"],
                "products": [
                    {
                        "id": first_item["product"]["id"]["$oid"],
                        "name": first_item["name"] + "與其它商品",
                        "imageUrl": first_item["product"]["images"][0]["expected_url"],
                        "quantity": 1,
                        "price": checkout_dict["total"],
                    }
                ],
            }
        ]

    def request(self, checkout_dict: Dict[str, Any]) -> Any:
        products = self._to_products(checkout_dict)
        total = checkout_dict["total"]
        body = {
            "amount": total,
            "currency": "TWD",
            "orderId": checkout_dict["order_number"],
            "packages": products,
            "redirectUrls": {
                "confirmUrl": self.callback_url,
                "cancelUrl": self.callback_url,
            },
        }
        json_str = json.dumps(body)
        path = "/v3/payments/request"
        url = f"{self.linepay_url}{path}"
        headers = self.build_headers(path, json_str)

        resp = requests.post(
            url=url,
            json=body,
            headers=headers,
        )
        self._raise_exception(resp)

        resp_json = resp.json()

        if not self.verify(resp_json):
            print(resp_json)
            raise LinePayException(
                error="linepay_verify_error",
                detail="",
            )

        return resp_json["info"]["paymentUrl"]

    def confirm(self, transaction_id: str, amount: int) -> Any:
        body = {"amount": amount, "currency": "TWD"}
        json_str = json.dumps(body)
        path = f"/v3/payments/{transaction_id}/confirm"
        url = f"{self.linepay_url}{path}"
        headers = self.build_headers(path, json_str)
        resp = requests.post(url=url, json=body, headers=headers)
        self._raise_exception(resp)
        return resp.json()

    def refund(self, transaction_id: str, refund_amount: int) -> Any:
        body = {"refundAmount": refund_amount}
        json_str = json.dumps(body)
        path = f"/v3/payments/{transaction_id}/refund"
        url = f"{self.linepay_url}{path}"
        headers = self.build_headers(path, json_str)
        resp = requests.post(
            url=url,
            json=body,
            headers=headers,
        )
        self._raise_exception(resp)
        return resp.json()

    def get_details(
        self, transaction_id: Optional[str] = None, order_id: Optional[str] = None
    ) -> Any:
        path = f"/v3/payments"
        query = ""
        if transaction_id is not None:
            query += "transactionId={}&".format(str(transaction_id))
        if order_id is not None:
            query += "orderId={}".format(order_id)
        if query.endswith("?") or query.endswith("&"):
            query = query[:-1]

        url = (
            f"{self.linepay_url}{path}?{query}"
            if query
            else f"{self.linepay_url}{path}"
        )
        headers = self.build_headers(path, query)
        resp = requests.get(
            url=url,
            headers=headers,
        )
        self._raise_exception(resp)
        return resp.json()



# ecpay
import io
from typing import Any, Dict, List, Tuple

import boto3
from bs4 import BeautifulSoup
from requests import Response

from fastel.collections import get_site_config
from fastel.config import SdkConfig
from fastel.exceptions import APIException
from fastel.payment.cryptors import MD5Cryptor
from fastel.payment.ecpay.models.checkout import CheckoutModel
from fastel.payment.ecpay.models.invoice import IssueB2BModel, IssueB2CModel
from fastel.payment.ecpay.models.logistics import (
    LogisticModel,
    LogisticsDataModel,
    LogisticsDetailModel,
)
from fastel.utils import requests

PAYMENT_SUBTYPE = {
    "default": "ALL",
    "credit": "Credit",
    "atm": "ATM",
    "web_atm": "WebATM",
    "cvs": "CVS",
    "barcode": "BARCODE",
}

CARRIER_TYPE_TABLE = {
    "0": "",
    "1": "1",
    "2": "2",
    "3": "3",
}


def ecpay_url() -> str:
    return SdkConfig.payment_host + "/ecpay"


def client_id() -> str:
    return SdkConfig.client_id


def client_secret() -> str:
    return SdkConfig.client_secret


def _replace_limit_name(name: str, limit: int) -> str:
    if len(name) > limit:
        return name[: limit - 3] + "..."
    return name


def checkout_request(
    checkout: Dict[str, Any],
) -> str:
    url = (
        ecpay_url()
        + f"/request?client_id={client_id()}&client_secret={client_secret()}"
    )
    name = "#".join([item["name"] for item in checkout.get("items", [])])
    item_name = _replace_limit_name(name=name, limit=400)
    common_props = {
        "MerchantTradeNo": checkout["order_number"]
        + "{:02d}".format(checkout["attempt"]),
        "ChoosePayment": PAYMENT_SUBTYPE.get(checkout["payment_subtype"], "ALL"),
        "TotalAmount": checkout["total"],
    }

    if checkout.get("is_custom", False):
        validated = CheckoutModel(
            **common_props,
            ItemName=checkout["custom_name"],
        )
    else:
        validated = CheckoutModel(
            **common_props,
            ItemName=item_name,
        )

    validated_dict = validated.dict(exclude_none=True)
    resp: Response = requests.post(url, json=validated_dict)
    resp_json = resp.json()
    assert isinstance(resp_json, dict)
    return resp_json.get("url", "")


def cvs_map(cart: Dict[str, Any]) -> str:
    url = (
        ecpay_url()
        + f"/logistics/map/request?client_id={client_id()}&client_secret={client_secret()}"
    )
    resp: Response = requests.post(
        url,
        json={
            "LogisticsType": "CVS",
            "LogisticsSubType": cart.get("logistics_subtype", "FAMIC2C"),
            "IsCollection": "Y"
            if cart.get("payment_subtype", "credit") == "cod"
            else "N",
        },
    )
    resp_json = resp.json()
    assert isinstance(resp_json, dict)
    return resp_json.get("url", "")


def logistics_create(order: Dict[str, Any]) -> Tuple[bool, Dict[str, Any]]:
    if order["total"] > 20000:
        return (False, {"validation_error": "物流訂單商品不能超過20000元新台幣"})
    common_props = dict(
        MerchantTradeNo=order["order_id"],
        GoodsName=f"訂單{order['order_id']}",
        GoodsAmount=order["total"],
        ReceiverName=_replace_limit_name(name=order["receiver_name"], limit=10),
        ReceiverEmail=order["receiver_email"],
        ReceiverCellPhone=order["receiver_phone"],
        ReceiverZipCode=order["receiver_zip"],
        ReceiverAddress=(
            order["receiver_city"]
            + order["receiver_district"]
            + order["receiver_address"]
        ),
    )

    default_sender_info = dict(
        SenderName="忻旅科技",
        SenderPhone="02-77295130",
        SenderZipCode="10361",
        SenderCellPhone="0900000000",
        SenderAddress="台北市大同區民權西路136號10樓之5",
    )

    sender_info = get_site_config(
        key="logistics_sender_info", default=default_sender_info
    )
    sender_info["SenderName"] = _replace_limit_name(
        name=sender_info["SenderName"], limit=10
    )
    data: LogisticModel
    if order["logistics_type"] == "HOME":
        data = LogisticModel(
            **common_props,
            **sender_info,
            Distance=("00" if order["receiver_city"] == "台北市" else "01"),
            Temperature="0001",
        )
    elif order["logistics_type"] == "CVS":
        data = LogisticModel(
            **common_props,
            **sender_info,
            LogisticsType="CVS",
            LogisticsSubType=order["logistics_subtype"],
            IsCollection="Y" if order["payment_subtype"] == "cod" else "N",
            CollectionAmount=order["total"] if order["payment_subtype"] == "cod" else 0,
            ReceiverStoreID=order["logistics_cvs_store_id"],
        )
    else:
        return (
            False,
            {"validation_error": f"no such logistics_type {order['logistics_type']}"},
        )

    url = (
        ecpay_url()
        + f"/logistics/create?client_id={client_id()}&client_secret={client_secret()}"
    )
    try:
        resp: Response = requests.post(url, json=data.dict(exclude_none=True))
        resp.raise_for_status()
        logistics_resp = resp.json()
        if logistics_resp.get("error", ""):
            raise APIException(
                status_code=500, error="server_error", detail="logistics return error"
            )
    except APIException:
        return (False, logistics_resp)
    except Exception:
        return (False, {"error": "response error"})
    return (True, logistics_resp)


def issue_invoice(checkout: Dict[str, Any]) -> Dict[str, Any]:
    if checkout.get("category", "B2C") == "B2C":
        url = (
            ecpay_url()
            + f"/B2C/invoice/issue?client_id={client_id()}&client_secret={client_secret()}"
        )
        invoice_data = generate_B2C_invoice_data(checkout)
    else:
        url = (
            ecpay_url()
            + f"/B2B/invoice/issue?client_id={client_id()}&client_secret={client_secret()}"
        )
        invoice_data = generate_B2B_invoice_data(checkout)
    try:
        resp: Response = requests.post(url, json=invoice_data)
        resp.raise_for_status()
        print("issue invoice")
        resp_json = resp.json()
    except Exception as error:
        return {"error": error}
    assert isinstance(resp_json, dict)
    return resp_json


def generate_B2C_invoice_item(
    items: List[Dict[str, Any]],
    extra_items: List[Dict[str, Any]],
    discount_items: List[Dict[str, Any]],
) -> List[Dict[str, Any]]:
    result = []
    for item in items:
        item_result = {
            "ItemName": _replace_limit_name(name=item.get("name", ""), limit=100),
            "ItemCount": item["config"].get("qty", 1),
            "ItemWord": item["config"]["extra_data"]
            and item["config"]["extra_data"].get("word", "份")
            or "份",
            "ItemPrice": item.get("price", 0),
            "ItemAmount": item.get("amount", 0),
        }
        result.append(item_result)
    # 計算運費
    for extra_item in extra_items:
        item_result = {
            "ItemName": _replace_limit_name(name=extra_item.get("name", ""), limit=100),
            "ItemCount": 1,
            "ItemWord": "份",
            "ItemPrice": extra_item.get("amount", 0),
            "ItemAmount": extra_item.get("amount", 0),
        }
        result.append(item_result)
    # 計算折扣金額
    for discount_item in discount_items:
        item_result = {
            "ItemName": _replace_limit_name(
                name=discount_item.get("name", ""), limit=100
            ),
            "ItemCount": 1,
            "ItemWord": "份",
            "ItemPrice": -discount_item.get("amount", 0),
            "ItemAmount": -discount_item.get("amount", 0),
        }
        result.append(item_result)
    return result


def generate_B2C_invoice_data(checkout: Dict[str, Any]) -> Dict[str, Any]:
    items = checkout.get("items", [])
    extra_items = checkout.get("extra_items", [])
    discount_items = checkout.get("discount_items", [])
    carrier_type = CARRIER_TYPE_TABLE[checkout.get("invoice_carrier_type", "1")]
    love_code = (
        checkout.get("invoice_donation", "0") == "1"
        and checkout.get("invoice_love_code", None)
        or None
    )
    carrier_num = (
        carrier_type in ["2", "3"] and checkout.get("invoice_carrier_num", "") or ""
    )
    invoice_data = {
        "RelateNumber": checkout.get("order_number", ""),
        "CustomerEmail": checkout.get("buyer_email", ""),
        "Print": "0",
        "Donation": checkout.get("invoice_donation", "0"),
        "LoveCode": love_code,
        "CarrierType": carrier_type,
        "CarrierNum": carrier_num,
        "TaxType": checkout.get("invoice_tax_type", "1"),
        "SalesAmount": checkout.get("total", 0),
        "Items": generate_B2C_invoice_item(items, extra_items, discount_items),
        "InvType": checkout.get("invoice_tax_type", "") == "4" and "08" or "07",
    }
    return IssueB2CModel.validate(invoice_data).dict(exclude_none=True)


def generate_B2B_invoice_item(
    items: List[Dict[str, Any]],
    extra_items: List[Dict[str, Any]],
    discount_items: List[Dict[str, Any]],
) -> List[Dict[str, Any]]:
    result = []
    seq = 1
    for item in items:
        item_result = {
            "ItemSeq": seq,
            "ItemName": _replace_limit_name(name=item.get("name", ""), limit=256),
            "ItemCount": item["config"].get("qty", 1),
            "ItemWord": item["config"]["extra_data"]
            and item["config"]["extra_data"].get("word", "份")
            or "份",
            "ItemPrice": item.get("unit_sales", 0),
            "ItemAmount": item.get("sales_amount", 0),
        }
        result.append(item_result)
        seq += 1
    # 計算運費
    for extra_item in extra_items:
        item_result = {
            "ItemSeq": seq,
            "ItemName": _replace_limit_name(name=extra_item.get("name", ""), limit=256),
            "ItemCount": 1,
            "ItemWord": "份",
            "ItemPrice": extra_item.get("sales_amount", 0),
            "ItemAmount": extra_item.get("sales_amount", 0),
        }
        result.append(item_result)
        seq += 1
    # 計算折扣金額
    for discount_item in discount_items:
        item_result = {
            "ItemSeq": seq,
            "ItemName": _replace_limit_name(
                name=discount_item.get("name", ""), limit=256
            ),
            "ItemCount": 1,
            "ItemWord": "份",
            "ItemPrice": -discount_item.get("sales_amount", 0),
            "ItemAmount": -discount_item.get("sales_amount", 0),
        }
        result.append(item_result)
        seq += 1
    return result


def generate_B2B_invoice_data(checkout: Dict[str, Any]) -> Dict[str, Any]:
    items = checkout.get("items", [])
    extra_items = checkout.get("extra_items", [])
    discount_items = checkout.get("discount_items", [])
    invoice_data = {
        "RelateNumber": checkout.get("order_number", ""),
        "CustomerIdentifier": checkout.get("invoice_uni_no", ""),
        "CustomerEmail": checkout.get("buyer_email", ""),
        "TaxType": checkout.get("invoice_tax_type", "1"),
        "SalesAmount": checkout.get("sales", 0),
        "TaxAmount": checkout.get("tax", 0),
        "TotalAmount": checkout.get("total", 0),
        "Items": generate_B2B_invoice_item(items, extra_items, discount_items),
        "InvType": checkout.get("invoice_tax_type", "") == "4" and "08" or "07",
    }
    return IssueB2BModel.validate(invoice_data).dict(exclude_none=True)


def fetch_html_img(html: str) -> str:
    soup = BeautifulSoup(html)
    img_tag = soup.find("img")
    if not img_tag:
        return ""
    else:
        return str(img_tag.get("src", ""))


def _generate_shipping_data(
    cryptor: MD5Cryptor, logistics_data: LogisticsDataModel
) -> Tuple[str, Dict[str, Any]]:
    if logistics_data.logistics_sub_type == "FAMIC2C":
        url = f"{SdkConfig.ecpay_logistics_host}/Express/PrintFAMIC2COrderInfo"
        data = {
            "MerchantID": SdkConfig.logistics_merchant_id,
            "AllPayLogisticsID": logistics_data.logistics_id,
            "CVSPaymentNo": logistics_data.cvs_payment_no,
        }
    elif logistics_data.logistics_sub_type == "UNIMARTC2C":
        url = f"{SdkConfig.ecpay_logistics_host}/Express/PrintUniMartC2COrderInfo"
        data = {
            "MerchantID": SdkConfig.logistics_merchant_id,
            "AllPayLogisticsID": logistics_data.logistics_id,
            "CVSPaymentNo": logistics_data.cvs_payment_no,
            "CVSValidationNo": logistics_data.cvs_validation_no,
        }
    elif logistics_data.logistics_sub_type == "HILIFEC2C":
        url = f"{SdkConfig.ecpay_logistics_host}/Express/PrintHILIFEC2COrderInfo"
        data = {
            "MerchantID": SdkConfig.logistics_merchant_id,
            "AllPayLogisticsID": logistics_data.logistics_id,
            "CVSPaymentNo": logistics_data.cvs_payment_no,
        }
    elif logistics_data.logistics_sub_type == "OKMARTC2C":
        url = f"{SdkConfig.ecpay_logistics_host}/Express/PrintOKMARTC2COrderInfo"
        data = {
            "MerchantID": SdkConfig.logistics_merchant_id,
            "AllPayLogisticsID": logistics_data.logistics_id,
            "CVSPaymentNo": logistics_data.cvs_payment_no,
        }
    elif logistics_data.logistics_sub_type in [
        "FAMI",
        "UNIMART",
        "UNIMARTFREEZE",
        "HILIFE",
        "TCAT",
        "ECAN",
    ]:
        url = f"{SdkConfig.ecpay_logistics_host}/helper/printTradeDocument"
        data = {
            "MerchantID": SdkConfig.logistics_merchant_id,
            "AllPayLogisticsID": logistics_data.logistics_id,
        }
    else:
        return "", {}
    check_mac = cryptor.encrypt(data)
    data["CheckMacValue"] = check_mac
    return url, data


# return img bytes, content_type and logistics id
def get_shipping_note(logistics_data: LogisticsDataModel) -> Tuple[bytes, str]:
    if (
        SdkConfig.logistics_merchant_id
        and SdkConfig.logistics_hash_iv
        and SdkConfig.logistics_hash_key
    ):
        cryptor = MD5Cryptor(
            hash_key=SdkConfig.logistics_hash_key, hash_iv=SdkConfig.logistics_hash_iv
        )
        url, form_data = _generate_shipping_data(
            cryptor=cryptor, logistics_data=logistics_data
        )
        resp: Response = requests.post(url, data=form_data)
        resp.raise_for_status()
        html = resp.content.decode()
        img_url = fetch_html_img(html)
        # is dirty in uri encode
        print("shipping note url:", img_url)
        img_url = img_url.replace("amp;", "")
        img_resp: Response = requests.get(img_url)
        img_resp.raise_for_status()
        img = img_resp.content
        content_type = img_resp.headers.get("Content-Type", "image/png")
        return img, content_type
    else:
        return b"", ""


def upload_shipping_note(logistics_data: LogisticsDataModel) -> None:
    shipping_note, content_type = get_shipping_note(logistics_data)
    print(
        "[FASTEL] ------------------------- upload_shipping_note ------------------------"
    )
    if shipping_note and content_type:
        s3 = boto3.client("s3", "ap-northeast-1")
        buf = io.BytesIO()
        buf.write(shipping_note)
        buf.seek(0)
        s3.upload_fileobj(
            buf,
            SdkConfig.s3_bucket,
            f"shipping_note/{logistics_data.logistics_id}",
            # shipping note must be private file
            ExtraArgs={"ContentType": content_type},
        )
        print("[FASTEL] ------------------------- SUCCESS ------------------------")
    else:
        print("[FASTEL] ------------------------- FAIL ------------------------")
        print(
            f"logistics_merchant_id: {SdkConfig.logistics_merchant_id}",
            f"logistics_hash_key: {SdkConfig.logistics_hash_key}",
            f"logistics_hash_iv: {SdkConfig.logistics_hash_iv}",
        )


def logistics_data_transfer(
    logistics_detail: LogisticsDetailModel,
) -> LogisticsDataModel:
    logistics_data = {
        "trade_no": logistics_detail.MerchantTradeNo,
        "logistics_id": logistics_detail.AllPayLogisticsID,
        "logistics_type": logistics_detail.LogisticsType,
        "logistics_sub_type": logistics_detail.LogisticsSubType,
        "cvs_payment_no": logistics_detail.CVSPaymentNo,
        "cvs_validation_no": logistics_detail.CVSValidationNo,
        "goods_amount": logistics_detail.GoodsAmount,
        "update_status_date": logistics_detail.UpdateStatusDate,
        "rtn_code": logistics_detail.RtnCode,
        "rtn_msg": logistics_detail.RtnMsg,
        "receiver_name": logistics_detail.ReceiverName,
        "receiver_phone": logistics_detail.ReceiverPhone,
        "receiver_cell_phone": logistics_detail.ReceiverCellPhone,
        "receiver_email": logistics_detail.ReceiverEmail,
        "receiver_address": logistics_detail.ReceiverAddress,
    }
    result = LogisticsDataModel.validate(logistics_data)
    return result



from typing import Any, Dict, List

import requests

from fastel.logistics.tcat.models import TcatLogsiticsData, TcatPickingRequestData


class TcatClient:
    default_headers = {
        "Content-Type": "application/json",
    }

    @property
    def url(self) -> str:
        if self.stage in ["stg", "STG"]:
            return "http://apitestsuda.southeastasia.cloudapp.azure.com:8081/api/Egs"
        return "https://api.suda.com.tw/api/Egs"

    def __init__(self, stage: str, client_id: str, client_token: str):
        self.stage = stage
        self._client_id = client_id
        self._client_token = client_token

    def _get(self, path: str) -> Any:
        full_url = f"{self.url}{path}"
        resp = requests.get(url=full_url, headers=self.default_headers)
        resp.raise_for_status()
        return resp

    def _post(self, path: str, data: Dict[str, Any]) -> Any:
        full_url = f"{self.url}{path}"
        resp = requests.post(url=full_url, json=data, headers=self.default_headers)
        resp.raise_for_status()
        return resp

    def query_postal_number(self, addresses: List[str]) -> Dict[str, Any]:
        address_array = [{"Search": address} for address in addresses]
        json_data = {
            "CustomerId": self._client_id,
            "CustomerToken": self._client_token,
            "Addresses": address_array,
        }
        resp = self._post("/ParsingAddress", json_data)
        return resp.json()  # type: ignore

    def create_logistics(self, data: TcatLogsiticsData) -> Dict[str, Any]:
        data_dict = data.dict(exclude_none=True)

        json_data = {
            "CustomerId": self._client_id,
            "CustomerToken": self._client_token,
            **data_dict,
        }
        resp = self._post("/PrintOBT", json_data)
        return resp.json()  # type: ignore

    def create_picking_list(self, data: TcatPickingRequestData) -> Dict[str, Any]:
        data_dict = data.dict(exclude_none=True)

        json_data = {
            "CustomerId": self._client_id,
            "CustomerToken": self._client_token,
            **data_dict,
        }
        resp = self._post("/PrintOBTByPickingList", json_data)
        return resp.json()  # type: ignore

    def download_logistics_pdf(self, FileNo: str) -> Dict[str, Any]:
        json_data = {
            "CustomerId": self._client_id,
            "CustomerToken": self._client_token,
            "FileNo": FileNo,
        }
        resp = self._post("/DownloadOBT", json_data)
        return {"file": resp.content}



from typing import Any, List, Literal, Optional

from pydantic import BaseModel, validator


class TcatOrderData(BaseModel):
    """
    ProductTypeId
    - 0001:一般食品
    - 0002:名特產/甜產
    - 0003:酒/油/醋/醬
    - 0004:穀物蔬果
    - 0005:水產/肉品
    - 0006:3C
    - 0007:家電
    - 0008:服飾配件
    - 0009:生活用品
    - 0010:美容彩妝
    - 0011:保健食品
    - 0012:醫療相關用品
    - 0013:寵物用品飼料
    - 0014:印刷品
    - 0015:其他
    """

    OBTNumber: str = ""  # 託運單號(列印類別欄位為01請填空)
    OrderId: str  # 客戶訂單號碼
    Thermosphere: Literal["0001", "0002", "0003"]  # 常溫/冷藏/冷凍
    Spec: Literal["0001", "0002", "0003", "0004"]  # 60-150cm
    ReceiptLocation: Literal["01", "02"] = "01"  # 到宅/站所
    ReceiptStationNo: Optional[str]  # 站所編號(若收付地點02必填)
    RecipientName: str
    RecipientTel: str
    RecipientMobile: str
    RecipientAddress: str
    SenderName: str
    SenderTel: str
    SenderMobile: str
    SenderZipCode: str
    SenderAddress: str
    ShipmentDate: str  # 出貨日期(yyyyMMdd/國定假日、周日不配送)
    DeliveryDate: str  # 希望配達日期(yyyyMMdd/國定假日、周日不配送)
    DeliveryTime: Literal["01", "02", "04"] = "04"  # 13前/14-18/不指定
    IsFreight: Literal["Y", "N"] = "N"  # 是否到付
    IsCollection: Literal["Y", "N"]  # 是否代收貨款
    CollectionAmount: int
    IsSwipe: Literal["Y", "N"] = "N"  # 是否允許刷卡
    IsDeclare: Literal["Y", "N"] = "N"  # 是否報值
    DeclareAmount: int = 0
    ProductTypeId: Literal[
        "0001",
        "0002",
        "0003",
        "0004",
        "0005",
        "0006",
        "0007",
        "0008",
        "0009",
        "0010",
        "0011",
        "0012",
        "0013",
        "0014",
        "0015",
    ] = "0015"
    ProductName: str
    Memo: Optional[str]

    @validator("SenderZipCode")
    def check_sender_zip(cls, v: str) -> str:
        if len(v) != 6:
            raise ValueError("zip code need in 6 digits")
        return v

    @validator("ProductName")
    def check_product_name(cls, v: str) -> str:
        if len(v) > 20:
            return v[:20]
        return v

    @validator("CollectionAmount")
    def verify_collection_amount(cls, v: int, values: Any) -> int:
        if values["IsCollection"] == "N":
            return 0
        return v


class TcatLogsiticsData(BaseModel):
    PrintType: str = "01"  # 列印類別
    PrintOBTType: Literal["01", "02", "03"] = "02"  # A4三模宅配
    Orders: List[TcatOrderData]


class TcatPickingBody(BaseModel):
    OrderId: str
    ProductTypeId: Literal[
        "0001",
        "0002",
        "0003",
        "0004",
        "0005",
        "0006",
        "0007",
        "0008",
        "0009",
        "0010",
        "0011",
        "0012",
        "0013",
        "0014",
        "0015",
    ] = "0015"
    ProductName: str
    Quantity: int
    Price: int
    Amount: int
    Column01: str = ""
    Column02: str = ""
    Column03: str = ""
    Column04: str = ""

    @validator("ProductName")
    def check_product_name(cls, v: str) -> str:
        if len(v) > 20:
            return v[:20]
        return v


class TcatPickingTitle(BaseModel):
    Column01: str = ""
    Column02: str = ""
    Column03: str = ""
    Column04: str = ""


class TcatPickingFooter(BaseModel):
    Row01: str = ""
    Row02: str = ""
    Row03: str = ""
    Row04: str = ""


class TcatPickingData(BaseModel):
    Title: TcatPickingTitle
    Body: List[TcatPickingBody]
    Footer: TcatPickingFooter


class TcatPickingOrderData(BaseModel):
    OBTNumber: str = ""
    Thermosphere: Literal["0001", "0002", "0003"]
    Spec: Literal["0001", "0002", "0003", "0004"]
    ReceiptLocation: Literal["01", "02"] = "01"
    ReceiptStationNo: Optional[str]
    RecipientName: str
    RecipientTel: str
    RecipientMobile: str
    RecipientAddress: str
    SenderName: str
    SenderTel: str
    SenderMobile: str
    SenderZipCode: str
    SenderAddress: str
    ShipmentDate: str
    DeliveryDate: str
    DeliveryTime: Literal["01", "02", "04"] = "04"
    IsFreight: Literal["Y", "N"] = "N"
    IsCollection: Literal["Y", "N"] = "N"
    CollectionAmount: int
    IsSwipe: Literal["Y", "N"] = "N"
    IsDeclare: Literal["Y", "N"] = "N"
    DeclareAmount: int = 0
    Memo: Optional[str]
    Detail: TcatPickingData

    @validator("SenderZipCode")
    def check_sender_zip(cls, v: str) -> str:
        if len(v) != 6:
            raise ValueError("zip code need in 6 digits")
        return v

    @validator("CollectionAmount")
    def verify_collection_amount(cls, v: int, values: Any) -> int:
        if values["IsCollection"] == "N":
            return 0
        return v


class TcatPickingRequestData(BaseModel):
    PrintType: str = "01"
    PrintOBTType: Literal["01"] = "01"
    Orders: List[TcatPickingOrderData]



import os
from typing import Any, Dict

# https://keys.revtel-api.com/certs.json
prod_cert = {
    "keys": [
        {
            "e": "AQAB",
            "kid": "bc36e546-f24a-4635-a77b-eecb20602504",
            "kty": "RSA",
            "n": "8oPMlKMnG9Uvyr8SMSLf5N7pl2ciEL-OWkAYngAzV1Y492cH8PWWFCibpiXU6iNlBbP_py2O6p8xfMHX1vGCF9uFU__iqc5RBbaCYL1kxczQbCt69tPLdyv3_6FNfMioc62Cym77rEVIa4uRLJl0TB_BJ89beCoL7BO6U1szGz3oVn-4igsc8GQRJOaZkZXY0JIBqB7dTHBgiq1R444ex_Tl1M6E9w45PxzowLOn1GWv6X5wyOSX1z-g60ErmDdxuu3N3lm2fJr9W-mjFgMvo4V9tBQmABaWO573I82IJymxt3C6B1g9Co9P1Thk5zMxtm33Om2FrizinUZbnQAtqQ",
        },
        {
            "e": "AQAB",
            "kid": "d4601977-6244-4a01-87dd-af0c1855886a",
            "kty": "RSA",
            "n": "yrIO_j-qbZfDlbLaTF-TQC38LVv8FVaXV4RPwNwSL_3QQA04HO1lnA8eXf2jV3RbI3WVey9jgDxjIidGAQQ0EJawXX7lPKWj56gDWBjPID7z-32DXicPww7vAgP8KD_GD00bwDnrnPCEveZQ9fAX-9zJpDNLRZlZ-GdgwOP55dIUFaOGeS1GP3jCBcy9auRzthBV4CO2Y_WzC69yL-4WgBPY_cIEc49vZ_Xyyo2mP36-fHzJc9eQ-moRczJ_hlCUe6DPfC99ojMFxXyd9SkD6BvTdnI5Bw-DPd4MCQj6KeSTuWTVfi2zsL2ooX0e8oLqmdhKqEB7T9LAr2efJkBGQQ",
        },
    ]
}

# https://keys.revtel-api.com/certs-stg.json
stg_cert = {
    "keys": [
        {
            "e": "AQAB",
            "kid": "df9885fa-942b-40c0-988c-9c3af76694bc",
            "kty": "RSA",
            "n": "x_-UkGLNQVRBXXW894iokDf7irGQ8rNPzsp-9N4ylVrkES93OL95E_rOdoK0Z8kCuwPOvKfc1QmgBFoJjMMaxI3zLDdDTl9QRIS1e1akecuAdzMj53X_t98Z2pgcT1paoDSkHh7qgYRKmt1xpU7fbKrogjdzqTv3vsnB3tQU2P1-9UJtH3-1BoAVhyiusFqXLH0o6Rp7drMbVYbvyj19nRwcBZz9gtO_bWyYGz0KUtFkm_vc31JmCARif7Tb4vc6FsmjGCgaQ9OSbJLgmYS7ZeVLFomyLKZuDeyAbS0rfzjC6Cf5heu6F2F44MdRoq-QK88nZ19fOpcJk1CkYPtySQ",
        },
        {
            "e": "AQAB",
            "kid": "1a7f2fbe-5112-4450-b7f1-27187d6030fb",
            "kty": "RSA",
            "n": "nPPHuzGJ8M9eZVr2f_CUrzFzyPQ0Ks9R31abO2B6qSOKQb_7aLQC7kOB02wWckyqpKhMRHTVbpKBJXYI1sga_iAaFfDyJ8-RVH3-hbpF1-_Bv7AteGJSZe2Etyi4kFXSZs2pDNOgUS6zvrkdQTlIqSst6MGJNKiaF1OpsmYlwAzJF37YAbrkuNOC1nbQorKkqQzSDa9667ZiEGoU65TGWP0FWwuzSBJGb8AOVNIEdIUaEdTrimoENzJOJR-RHeRGPLpU_Fe7TRj_RLyixDB6Hp_ZyYHW3Su7N3YBu2vZnp42d29E1UMFCU6k_5uoWR6rvosxCL1FkfaoXvpQ78oDmQ",
        },
    ]
}


dsa_cert = {
    "keys": [
        {
            "e": "AQAB",
            "kid": "428ab35b-9743-4034-bc95-87035411056b",
            "kty": "RSA",
            "n": "oAadb4E0B6vniuTjfFpybq314GkHMPVCxgQ-6a_a0Xo_HZrTsb-1ANC4GriFcJdswMJWRDO22RLE-1WTRyoW68HZ9YKzu6ZaRViAX4ihJXPeTF0uiqf9A9wAuSt5ZzLB4dVh9QJavwM1IaaxIzMhfeM1ykAd6bj1RazO9FatfhqJJbqsbCi_Ze4nUq4hrFsVkMXJBNew7yEBc6_NKUuLDYTOhdT-w9Rj6BM152Um-lEg27ENZCxJ5i_AhStw4in-Pk7V4yBPuscAqNiUSUUYc28ZmsX8T9KUBOPmiOAmbbYHmP7KZx3Gf5xqgxjL8NfMseOK5S9G-Cjsgy5eJYOM4Q",
        }
    ]
}


class SdkConfigCls:
    sdk_client: str = ""
    stage: str = ""
    client_secret: str = ""
    conversion_id: str = ""
    conversion_token: str = ""

    logistics_merchant_id: str = ""
    logistics_hash_key: str = ""
    logistics_hash_iv: str = ""
    s3_bucket: str = ""

    extra_config: Dict[str, Any] = {}

    @property
    def provider(self) -> str:
        parts = self.sdk_client.split(".")
        return parts[0]

    @property
    def client_id(self) -> str:
        parts = self.sdk_client.split(".")
        return parts[1]

    @property
    def package(self) -> str:
        parts = self.sdk_client.split(".")
        return parts[-1]

    @property
    def is_service(self) -> bool:
        return True if self.provider == "service" else False

    @property
    def auth_host(self) -> str:
        if self.stage in ["stg", "dev"] and self.package == "revtel":
            return "https://auth-stg.revtel-api.com/v4"
        elif self.stage in ["prod", "production"] and self.package == "revtel":
            return "https://auth.revtel-api.com/v4"
        elif self.stage == "prod" and self.package == "DSA":
            return "https://auth.dsa-automation.com.tw/v4"
        else:
            return ""

    @property
    def ntfn_host(self) -> str:
        if self.stage == "stg" and self.package == "revtel":
            return "https://notification-stg.revtel-api.com/v4"
        elif self.stage in ["prod", "production"] and self.package == "revtel":
            return "https://notification.revtel-api.com/v4"
        elif self.stage == "dev" and self.package == "revtel":
            return "https://notification-dev.revtel-api.com/v4"
        else:
            return ""

    @property
    def payment_host(self) -> str:
        if self.stage in ["stg", "dev"] and self.package == "revtel":
            return "https://payment-stg.revtel-api.com/v3"
        elif self.stage in ["prod", "production"] and self.package == "revtel":
            return "https://payment.revtel-api.com/v3"
        else:
            return ""

    @property
    def ecpay_logistics_host(self) -> str:
        if self.stage in ["stg", "dev"]:
            return "https://logistics-stage.ecpay.com.tw"
        elif self.stage in ["prod", "production"]:
            return "https://logistics.ecpay.com.tw"
        else:
            return ""

    @property
    def public_key(self) -> Dict[str, Any]:
        if self.stage == "stg" and self.package == "revtel":
            return stg_cert
        elif self.stage in ["prod", "production"] and self.package == "revtel":
            return prod_cert
        elif self.stage in ["prod", "production"] and self.package == "DSA":
            return dsa_cert
        else:
            return {}

    def put_extras(self, **kwargs: Any) -> None:
        for key, value in kwargs.items():
            self.extra_config[key] = value

    def get_extra(self, key: str, default_value: Any = None) -> Any:
        return self.extra_config.get(key, default_value)


SdkConfig = SdkConfigCls()


def sdk_auto_config(sdk_client: str = "") -> None:
    global SdkConfig
    print("SdkConfig", SdkConfig)

    def get_env_or_raise(key: str, raise_exception: bool = True) -> str:
        value: str = os.environ.get(key, "")
        if value == "" and raise_exception:
            raise AttributeError(f"[sdk_auto_config] {key} not found")
        return value

    # if not sdk_client initial app by revtel
    if not sdk_client:
        client_id = get_env_or_raise("CLIENT_ID")
        sdk_client = f"app.{client_id}.revtel"

    # common env vars
    SdkConfig.stage = get_env_or_raise("STAGE")

    SdkConfig.sdk_client = sdk_client

    if not SdkConfig.is_service:
        SdkConfig.client_secret = get_env_or_raise("CLIENT_SECRET")
        # for facebook conversion vars
        SdkConfig.conversion_id = get_env_or_raise(
            "CONVERSION_ID", raise_exception=False
        )
        SdkConfig.conversion_token = get_env_or_raise(
            "CONVERSION_TOKEN", raise_exception=False
        )
        # project env vars with ecpay logistics secret
        SdkConfig.logistics_merchant_id = get_env_or_raise(
            "LOGISTICS_MERCHANT_ID", False
        )
        SdkConfig.logistics_hash_key = get_env_or_raise("LOGISTICS_HASH_KEY", False)
        SdkConfig.logistics_hash_iv = get_env_or_raise("LOGISTICS_HASH_IV", False)
        SdkConfig.s3_bucket = get_env_or_raise("S3_BUCKET", False)



from typing import List, Literal

from pydantic import BaseModel


class LinePayRequestProduct(BaseModel):
    id: str
    name: str
    imageUrl: str
    quantity: int
    price: int


class LinePayRequestPackage(BaseModel):
    id: str
    amount: int
    products: List[LinePayRequestProduct]


class LinePayRedirectUrl(BaseModel):
    confirmUrl: str
    cancelUrl: str


class LinePayRequestModel(BaseModel):
    amount: int
    currency: Literal["USD", "JPY", "TWD", "THB"] = "TWD"
    orderId: str
    packages: List[LinePayRequestPackage]
    redirectUrls: LinePayRedirectUrl
